package server

import (
	"fmt"
	"net/http"
	"time"

	"github.com/go-chi/chi/v5/middleware"
	"go.opentelemetry.io/contrib/instrumentation/net/http/otelhttp"
)

func (s *UserServer) MiddlewareLogging(next http.Handler) http.Handler {
	fn := func(w http.ResponseWriter, r *http.Request) {
		ww := middleware.NewWrapResponseWriter(w, r.ProtoMajor)

		t1 := time.Now()
		defer func(ww middleware.WrapResponseWriter) {
			s.Logger.Info("Served request.",
				"status", ww.Status(),
				"method", r.Method,
				"path", r.URL.Path,
				"duration", time.Since(t1),
				"written", ww.BytesWritten())
		}(ww)
		next.ServeHTTP(ww, r)
	}
	return http.HandlerFunc(fn)
}

func (s *UserServer) MiddlewareTracing(next http.Handler) http.Handler {
	fn := func(w http.ResponseWriter, r *http.Request) {
		h := otelhttp.NewHandler(next, fmt.Sprintf("%s %s", r.Method, r.URL.Path))
		h.ServeHTTP(w, r)
	}
	return http.HandlerFunc(fn)
}
